package nemosofts.online.live.activity;

import android.app.Application;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.nemosofts.EnvatoProduct;
import androidx.nemosofts.LauncherListener;
import androidx.nemosofts.theme.ColorUtils;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;

import nemosofts.online.live.BuildConfig;
import nemosofts.online.live.R;
import nemosofts.online.live.asyncTask.LoadAbout;
import nemosofts.online.live.asyncTask.LoadLogin;
import nemosofts.online.live.callback.Callback;
import nemosofts.online.live.callback.Method;
import nemosofts.online.live.interfaces.AboutListener;
import nemosofts.online.live.interfaces.LoginListener;
import nemosofts.online.live.utils.IfSupported;
import nemosofts.online.live.utils.helper.DBHelper;
import nemosofts.online.live.utils.helper.Helper;
import nemosofts.online.live.utils.helper.SPHelper;

public class LauncherActivity extends AppCompatActivity implements LauncherListener {

    private Helper helper;
    private SPHelper spHelper;
    private DBHelper dbHelper;
    private ProgressBar pb;
    Application application;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_launcher);
        IfSupported.hideStatusBar(this);

        helper = new Helper(this);
        spHelper = new SPHelper(this);
        dbHelper = new DBHelper(this);

        pb = findViewById(R.id.pb_splash);

        findViewById(R.id.rl_splash).setBackgroundColor(ColorUtils.colorBg(this));

        loadAboutData();
    }

    private void loadAboutData() {
        if (helper.isNetworkAvailable()) {
            LoadAbout loadAbout = new LoadAbout(LauncherActivity.this, new AboutListener() {
                @Override
                public void onStart() {
                    pb.setVisibility(View.VISIBLE);
                }

                @Override
                public void onEnd(String success, String verifyStatus, String message){
                    pb.setVisibility(View.GONE);
                    if (success.equals("1")){
                        if (!verifyStatus.equals("-1") && !verifyStatus.equals("-2")){
                            dbHelper.addtoAbout();
                            setSaveData();
                        } else {
                            errorDialog(getString(R.string.err_unauthorized_access), message);
                        }
                    } else {
                        errorDialog(getString(R.string.err_server), getString(R.string.err_server_not_connected));
                    }
                }
            });
            loadAbout.execute();
        } else {
            if (Boolean.TRUE.equals(spHelper.getIsFirst())) {
                errorDialog(getString(R.string.err_internet_not_connected), getString(R.string.err_try_internet_connected));
            } else {
                try {
                    dbHelper.getAbout();
                    setSaveData();
                } catch (Exception e) {
                    e.printStackTrace();
                    errorDialog(getString(R.string.err_internet_not_connected), getString(R.string.err_try_internet_connected));
                }
            }
        }
    }

    private void setSaveData() {
        new EnvatoProduct(this, this).execute();
    }

    private void loadSettings() {
        if (Boolean.TRUE.equals(Callback.isAppUpdate) && Callback.app_new_version != BuildConfig.VERSION_CODE){
            openDialogActivity(Callback.DIALOG_TYPE_UPDATE);
        } else if(Boolean.TRUE.equals(Callback.isMaintenance)){
            openDialogActivity(Callback.DIALOG_TYPE_MAINTENANCE);
        } else {
            if (Boolean.TRUE.equals(spHelper.getIsFirst())) {
                if (Boolean.TRUE.equals(Callback.isLogin)){
                    openSignInActivity();
                } else {
                    spHelper.setIsFirst(false);

                    application = getApplication();
                    ((MyApplication) application).loadAd(LauncherActivity.this);

                    openMainActivity();
                }
            } else {
                application = getApplication();
                ((MyApplication) application).loadAd(LauncherActivity.this);

                if (Boolean.FALSE.equals(spHelper.getIsAutoLogin())) {
                    openMainActivity();
                } else {
                    if (spHelper.getLoginType().equals(Method.LOGIN_TYPE_GOOGLE)) {
                        FirebaseUser currentUser = FirebaseAuth.getInstance().getCurrentUser();
                        if (currentUser != null) {
                            loadLogin(Method.LOGIN_TYPE_GOOGLE, spHelper.getAuthID());
                        } else {
                            spHelper.setIsAutoLogin(false);
                            openMainActivity();
                        }
                    } else {
                        loadLogin(Method.LOGIN_TYPE_NORMAL, "");
                    }
                }
            }
        }
    }

    private void loadLogin(final String loginType, final String authID) {
        if (helper.isNetworkAvailable()) {
            LoadLogin loadLogin = new LoadLogin(new LoginListener() {
                @Override
                public void onStart() {
                    pb.setVisibility(View.VISIBLE);
                }

                @Override
                public void onEnd(String success, String loginSuccess, String message, String user_id, String user_name, String user_gender, String user_phone, String profile_img) {
                    pb.setVisibility(View.GONE);
                    if (success.equals("1") && (!loginSuccess.equals("-1"))) {
                        spHelper.setLoginDetails(user_id, user_name, user_phone, spHelper.getEmail(), user_gender, profile_img, authID, spHelper.getIsRemember(), spHelper.getPassword(), loginType);
                        spHelper.setIsLogged(true);
                    }
                    openMainActivity();
                }
            }, helper.getAPIRequest(Method.METHOD_LOGIN, 0,"","","","","","",spHelper.getEmail(),"","",spHelper.getPassword(),authID,loginType,null));
            loadLogin.execute();
        } else {
            Toast.makeText(LauncherActivity.this, getString(R.string.err_internet_not_connected), Toast.LENGTH_SHORT).show();
            spHelper.setIsAutoLogin(false);
            openMainActivity();
        }
    }

    private void openMainActivity() {
        pb.setVisibility(View.VISIBLE);
        new Handler().postDelayed(() -> {
            Intent intent = null;
            if (!((MyApplication) application).appOpenAdManager.isAdAvailable() || (Callback.isAppOpenAdShown && !((MyApplication) application).appOpenAdManager.isShowingAd)) {
                intent = new Intent(LauncherActivity.this, MainActivity.class);
                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                startActivity(intent);
                finish();
            } else if(((MyApplication) application).appOpenAdManager.isAdAvailable() && !((MyApplication) application).appOpenAdManager.isShowingAd) {
                ((MyApplication) application).appOpenAdManager.showAdIfAvailable(LauncherActivity.this);
            }
        }, 5500);
    }


    private void openSignInActivity() {
        Intent intent = new Intent(LauncherActivity.this, SignInActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        intent.putExtra("from", "");
        startActivity(intent);
        finish();
    }

    private void openDialogActivity(String type) {
        Intent intent = new Intent(LauncherActivity.this, DialogActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        intent.putExtra("from", type);
        startActivity(intent);
        finish();
    }

    @Override
    public void onStartPairing() {
        pb.setVisibility(View.VISIBLE);
    }

    @Override
    public void onConnected() {
        pb.setVisibility(View.GONE);
        loadSettings();
    }

    @Override
    public void onUnauthorized(String message) {
        pb.setVisibility(View.GONE);
        errorDialog(getString(R.string.err_unauthorized_access), message);
    }

    @Override
    public void onError() {
        pb.setVisibility(View.GONE);
        errorDialog(getString(R.string.err_server), getString(R.string.err_server_not_connected));
    }

    private void errorDialog(String title, String message) {
        final AlertDialog.Builder alertDialog = new AlertDialog.Builder(LauncherActivity.this, R.style.dialogTheme);
        alertDialog.setTitle(title);
        alertDialog.setMessage(message);
        alertDialog.setCancelable(false);
        if (title.equals(getString(R.string.err_internet_not_connected)) || title.equals(getString(R.string.err_server_not_connected))) {
            alertDialog.setNegativeButton(getString(R.string.retry), (dialog, which) -> loadAboutData());
        }
        alertDialog.setPositiveButton(getString(R.string.exit), (dialog, which) -> finish());
        alertDialog.show();
    }

    @Override
    public void onDestroy() {
        try {
            dbHelper.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        super.onDestroy();
    }
}